/**----------------------------------------------------
 * File:      SVGEditor.java
 * Package:   at.hoertler.svgeditor
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 *
 */
package at.hoertler.svgeditor;

import java.awt.Dimension;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;


public class SVGEditor {
  private static Editor editor = null;
  private static String configfile = "config/svgEditor.conf";
  private static String propertiefile = "resources/svgEditorStrings_de.properties";
  private static String version = "SVG Editor v0.0.1";

  /**
   * Constructor for EditorMain.
   *
   * @param fileName String
   */
  public SVGEditor(String fileName) {
    this.editor = new Editor();
    JFrame mainFrame = new JFrame();
    mainFrame.setTitle(Reader.getPropertyValueAsString(
            Reader.ConfigProperty.applicationName));
    this.editor.init(mainFrame, fileName);
    
    
    mainFrame.setSize(new Dimension(Reader.
            getPropertyValueAsInt(Reader.ConfigProperty.applicationWidth),
                                    Reader.
            getPropertyValueAsInt(Reader.ConfigProperty.applicationHeight)));
    mainFrame.setResizable(true);
    mainFrame.setLocation(350, 150);
    mainFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);//.EXIT_ON_CLOSE);
    //this.editor.init(mainFrame, fileName);
  }

  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) {
    final String fileName = parseCmdParam(args);

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {

        new SVGEditor(fileName);
      }
    });
  }

  /**
   * This method parses the start parameters and initializes the ConfigReader.
   *
   * @param args start parameters.
   */
  private static String parseCmdParam(final String[] args) {
    Options options = new Options();

    options.addOption("v", "version", false, "print version info");
    options.addOption("c", "conf", true,
                      "specify the path to the configuration file (default = " + configfile + ").");
    options.addOption("p", "prop", true,
                      "specify the path to the properties file (default = " + propertiefile + ").");
    options.addOption("h", "help", false, "print help usage");

    CommandLineParser parser = new BasicParser();
    HelpFormatter formatter = new HelpFormatter();
    CommandLine cmd = null;

    try {
      cmd = parser.parse(options, args);
      if (cmd.hasOption("c")) {
        configfile = cmd.getOptionValue("c");
      }
      Reader.setConfigurationFile(Reader.ReaderType.ConfigReader, configfile);

      if (cmd.hasOption("p")) {
        propertiefile = cmd.getOptionValue("p");
      } else if (Reader.getPropertyValueAsString(
              Reader.ConfigProperty.editorStringsFile) != null
              && Reader.getPropertyValueAsString(
              Reader.ConfigProperty.editorStringsFile) != "") {
        propertiefile = Reader.getPropertyValueAsString(
                Reader.ConfigProperty.editorStringsFile);
      }
      Reader.setConfigurationFile(Reader.ReaderType.StringReader, propertiefile);

      if (cmd.hasOption("h")) {
        formatter.printHelp("SVGEditor", options);
        System.exit(0);
      }

      if (cmd.hasOption("v")) {
        String v = Reader.
                getPropertyValueAsString(Reader.ConfigProperty.version);
        if (v != "") {
          version = v;
        }
        System.out.println(version);
        System.exit(0);
      }
      if(cmd.getArgs().length>0 && (cmd.getArgs()[0].toLowerCase().endsWith(".pdf") || cmd.getArgs()[0].toLowerCase().endsWith(".svg")))
        return cmd.getArgs()[0];
      return "";
    } catch (ParseException ex) {
      System.err.
              println("An error occured while parsing the start parameters. /n/n" + ex.
              toString());
    }
    return "";
  }
}
