 /**----------------------------------------------------
 * File:      HelperFunction.java
 * Package:   at.hoertler.svgeditor.common
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 *
 */
package at.hoertler.svgeditor.common;

import com.sun.imageio.plugins.png.PNGMetadata;
import java.awt.Point;
import java.awt.geom.Line2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.stream.ImageOutputStream;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class HelperFunction {

  /**
   * This method calculates the translation values between two points.
   *
   * @param from The point where the translation starts.
   * @param to   The point where the translation ends.
   *
   * @return translation point.
   */
  public static Point calculateTranslationPoint(Point from, Point to) {
    if (from == null || to == null)
      return new Point(0, 0);
    return new Point(to.x - from.x, to.y - from.y);
  }

  /**
   * This method calculates the intersection point of two Line2D objects
   *
   * @param l First Line2D
   * @param m Second Line2D
   *
   * @return Point of intersection
   */
  public static Point intersectLines(Line2D l, Line2D m) {
    int x1 = (int) l.getX1();
    int x2 = (int) l.getX2();
    int x3 = (int) m.getX1();
    int x4 = (int) m.getX2();
    int y1 = (int) l.getY1();
    int y2 = (int) l.getY2();
    int y3 = (int) m.getY1();
    int y4 = (int) m.getY2();

    // dividend
    int zx = (x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2) * (x3 * y4 - y3 * x4);
    int zy = (x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4);

    // divisor
    int n = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);

    if (n == 0)
      return new Point(0, 0);
    // Intersectionpoint
    int x = zx / n;
    int y = zy / n;

    return new Point(x, y);
  }
  
  public static String getPNGMetaData(File file, String key) {
    try {
      RandomAccessFile f = new RandomAccessFile(file, "r");
      long longlength = f.length();
      int length = (int) longlength;
      if (length != longlength) throw new IOException("File size >= 2 GB");

      // Read file and return data
      byte[] data = new byte[length];
      f.readFully(data);
     return HelperFunction.readCustomData(data, "SVGCode");
    } catch (IOException ex) {
      Logger.getLogger(HelperFunction.class.getName()).log(Level.SEVERE, null, ex);
    }
    return "";
  }

  public static  byte[] writeCustomData(BufferedImage buffImg, String key, String value)
          throws Exception {
    ImageWriter writer = ImageIO.getImageWritersByFormatName("png").next();

    ImageWriteParam writeParam = writer.getDefaultWriteParam();
    ImageTypeSpecifier typeSpecifier = ImageTypeSpecifier.
            createFromBufferedImageType(BufferedImage.TYPE_INT_RGB);

    //adding metadata
    IIOMetadata metadata = writer.getDefaultImageMetadata(typeSpecifier,
                                                          writeParam);

    IIOMetadataNode textEntry = new IIOMetadataNode("tEXtEntry");
    textEntry.setAttribute("keyword", key);
    textEntry.setAttribute("value", value);

    IIOMetadataNode text = new IIOMetadataNode("tEXt");
    text.appendChild(textEntry);

    IIOMetadataNode root = new IIOMetadataNode("javax_imageio_png_1.0");
    root.appendChild(text);

    metadata.mergeTree("javax_imageio_png_1.0", root);

    //writing the data
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    ImageOutputStream stream = ImageIO.createImageOutputStream(baos);
    writer.setOutput(stream);
    writer.write(metadata, new IIOImage(buffImg, null, metadata), writeParam);
    stream.close();

    return baos.toByteArray();
    //return ImageIO.read(new ByteArrayInputStream(baos.toByteArray()));
  }

  public static String readCustomData(/*BufferedImage img*/byte[] imageData, String key)
          throws IOException {
    /*ByteArrayOutputStream baos=new ByteArrayOutputStream();
    ImageIO.write(img, "png", baos );
    byte[] imageData=baos.toByteArray();*/
    
    ImageReader imageReader = ImageIO.getImageReadersByFormatName("png").next();

    imageReader.setInput(ImageIO.
            createImageInputStream(new ByteArrayInputStream(imageData)), true);

    // read metadata of first image
    IIOMetadata metadata = imageReader.getImageMetadata(0);

    //this cast helps getting the contents
    PNGMetadata pngmeta = (PNGMetadata) metadata;
    NodeList childNodes = pngmeta.getStandardTextNode().getChildNodes();

    for (int i = 0; i < childNodes.getLength(); i++) {
      Node node = childNodes.item(i);
      String keyword = node.getAttributes().getNamedItem("keyword").
              getNodeValue();
      String value = node.getAttributes().getNamedItem("value").getNodeValue();
      if (key.equals(keyword))
        return value;
    }
    return null;
  }
  public static String readCustomData(BufferedImage img, String key)
          throws IOException {
    ByteArrayOutputStream baos=new ByteArrayOutputStream();
    ImageIO.write(img, "png", baos );
    byte[] imageData=baos.toByteArray();
    
    ImageReader imageReader = ImageIO.getImageReadersByFormatName("png").next();

    imageReader.setInput(ImageIO.
            createImageInputStream(new ByteArrayInputStream(imageData)), true);

    // read metadata of first image
    IIOMetadata metadata = imageReader.getImageMetadata(0);

    //this cast helps getting the contents
    PNGMetadata pngmeta = (PNGMetadata) metadata;
    NodeList childNodes = pngmeta.getStandardTextNode().getChildNodes();

    for (int i = 0; i < childNodes.getLength(); i++) {
      Node node = childNodes.item(i);
      String keyword = node.getAttributes().getNamedItem("keyword").
              getNodeValue();
      String value = node.getAttributes().getNamedItem("value").getNodeValue();
      if (key.equals(keyword))
        return value;
    }
    return null;
  }
}
