 /**----------------------------------------------------
 * File:      HistoryManager.java
 * Package:   at.hoertler.svgeditor.data
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 * This class represents the chronic of actions that 
 * were made by the user.
 */
package at.hoertler.svgeditor.data;

import at.hoertler.svgeditor.svg.elements.SVGElement;
import at.hoertler.svgeditor.svg.elements.SVGElement.Mode;
import java.util.LinkedList;
import java.util.Vector;


public class HistoryManager {
  private LinkedList<Vector<SVGElement>> history = null;
  private int histIndex;
  private int maxHistSize = 30;
  
  public HistoryManager() {
    this.history = new LinkedList<Vector<SVGElement>>();
    this.history.add(new Vector<SVGElement>());
    this.histIndex = 0;
  }
  
  /**
   * Clears the history and sets SVGElements.
   *
   * @param elems Elements to be set.
   */
  public void setElements(Vector<SVGElement> elems) {
    this.history.clear();
    this.history.addFirst(elems);
  }
  
   /**
   * Gets the SVGElements of the current history Point
   *
   * @return A vector with the SVGElements.
   */
  public Vector<SVGElement> getElements() {
    return this.history.get(histIndex);
  }
  
  public Vector<SVGElement> getHistoryPoint() {
    Vector<SVGElement> ret = new Vector<SVGElement>();
    for(SVGElement e : this.getElements())
      ret.add(e.clone());
    return ret;
  }
  
  public boolean CreateNewHistoryPoint(Vector<SVGElement> elems) {
    //Vector<SVGElement> tmp = this.history.get(this.histIndex);
    for(int i = 0; i < this.histIndex; i++)
      this.history.removeFirst();
    this.histIndex = 0;
    this.history.addFirst(new Vector<SVGElement>());
    for(SVGElement e : elems) {
      Mode m = e.getMode();
      if(m.equals(Mode.TextEdit))
        e.setMode(Mode.Edit);
      SVGElement elem = e.clone();
      this.history.getFirst().add(elem);
      if(m.equals(Mode.TextEdit))
        e.setMode(Mode.TextEdit);
    }
    if(this.history.size() > this.maxHistSize)
      this.history.removeLast();
    return true;
  }
  
  public boolean undo() {
    if(!undoPossible())
      return false;
    histIndex++;
    return true;
  }
  
  public boolean redo() {
    if(!redoPossible())
      return false;
    histIndex--;
    return true;
  }
  
  public boolean undoPossible() {
    if(this.history.size()-1 > this.histIndex)
      return true;
    return false;
  }
  
  public boolean redoPossible() {
    if(this.histIndex>0)
      return true;
    return false;
  }

  public void clearHistory() {
    this.histIndex = 0;
    this.history.clear();
    this.history.add(new Vector<SVGElement>());
  }
}
