 /**----------------------------------------------------
 * File:      AlignmentMenuItem.java
 * Package:   at.hoertler.svgeditor.display.menubar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 * This class represents the graphical user interface to align 
 * all or just a subset of Elements on the canvas.
 */
package at.hoertler.svgeditor.display.menubar;

import at.hoertler.svgeditor.action.ActionManager;
import at.hoertler.svgeditor.event.ActionPerformedEvent;
import java.awt.Color;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JMenuItem;


public class AlignmentMenuItem extends JMenuItem{
  public AlignmentMenuItem() {
    this.removeAll();
    this.setPreferredSize(new Dimension(145,125));
    GridLayout gridLayout = new GridLayout(4,4);
    gridLayout.setHgap(0);
    gridLayout.setVgap(0);
    //this.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
    this.setLayout(gridLayout);
    this.getInsets().set(0, 0, 0, 0);
    this.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
    this.getMargin().set(0, 0, 0, 0);
    createButtons();
    
  }
  
  private JButton createButton() {
    JButton b = new JButton();
    b.setBorder(BorderFactory.createEmptyBorder());
    b.setPreferredSize(new Dimension(30,30));
    b.setMinimumSize(new Dimension(30,30));
    b.setBackground(Color.WHITE);
    b.setBorderPainted(false);
    b.setFocusable(false);
    b.setIconTextGap(0);
    b.getMargin().set(0, 0, 0, 0);
    return b;
  }
  
  private void createButtons() {
    JButton but = null;
    but = createButton();
    but.setEnabled(false);
    this.add(but);

    
    /*
     * Left
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        int[] px = {22,7,22};
        int[] py = {7,15,22};
        g.fillPolygon(px, py, 3);
        g.drawLine(7, 7, 7, 22);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.left));
      }
    });
    this.add(but);
    
    /*
     * center_h
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        int[] px1 = {7,15,7};
        int[] py1 = {7,15,22};
        g.fillPolygon(px1, py1, 3);
        
        int[] px2 = {22,15,22};
        int[] py2 = {7,15,22};
        g.fillPolygon(px2, py2, 3);
        g.drawLine(15, 7, 15, 22);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.center_h));
      }
    });
    this.add(but);
    
    /*
     * right
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        int[] px = {7,22,7};
        int[] py = {7,15,22};
        g.fillPolygon(px, py, 3);
        g.drawLine(22, 7, 22, 22);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.right));
      }
    });
    this.add(but);
    
    /*
     * Top
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        int[] py = {22,7,22};
        int[] px = {7,15,22};
        g.fillPolygon(px, py, 3);
        g.drawLine(7, 7, 22, 7);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.top));
      }
    });
    this.add(but);
    
    /*
     * Left_Up
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(4, 4, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.left_up));
      }
    });
    this.add(but);
    
    /*
     * Center_Up
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(11, 4, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.center_up));
      }
    });
    this.add(but);
    
    /*
     * Right_Up
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(17, 4, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.right_up));
      }
    });
    this.add(but);
    
    /*
     * Center_V
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        int[] py1 = {7,15,7};
        int[] px1 = {7,15,22};
        g.fillPolygon(px1, py1, 3);
        
        int[] py2 = {24,15,24};
        int[] px2 = {7,15,22};
        g.fillPolygon(px2, py2, 3);
        g.drawLine(7, 15, 22, 15);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.center_v));
      }
    });
    this.add(but);
    
    /*
     * Left_Center
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(4, 10, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.left_center));
      }
    });
    this.add(but);
    
    /*
     * Center_Center
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(11, 10, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.center_center));
      }
    });
    this.add(but);
    
    /*
     * Right_Center
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(17, 10, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.right_center));
      }
    });
    this.add(but);
    
    /*
     * Bottom
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        int[] py = {7,22,7};
        int[] px = {7,15,22};
        g.fillPolygon(px, py, 3);
        g.drawLine(7, 22, 22, 22);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.bottom));
      }
    });
    this.add(but);
    
   /*
     * Left_Bottom
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(4, 17, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.left_bottom));
      }
    });
    this.add(but);
    
    /*
     * Center_Bottom
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(11, 17, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.center_bottom));
      }
    });
    this.add(but);
    
    /*
     * Right_Bottom
     */
    but = createButton();
    but.setIcon(new Icon() {
      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        g.fillOval(17, 17, 8, 8);
      }

      @Override
      public int getIconWidth() { return 15; }

      @Override
      public int getIconHeight() { return 15; }
    });
    but.addActionListener(new ActionListener() {

      @Override
      public void actionPerformed(ActionEvent e) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                ActionPerformedEvent.Alignment.right_bottom));
      }
    });
    this.add(but);
  }
  
  
}
