 /**----------------------------------------------------
 * File:      ColorOptionPopUp.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Popup used for color selection (linecolor + fillcolor)
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionChangedEvent;
import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.util.Map.Entry;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JPanel;

public class ColorOptionPopUp
        extends OptionPopUp {

  public ColorOptionPopUp(OptionChangedEvent.Option o, Object v) {
    super(o, v);
  }

  @Override
  protected void init() {
    this.removeAll();
    int cellSize = 25;
    int offset = 255 / 7;
    FlowLayout fl = new FlowLayout(FlowLayout.CENTER, 0, 0);
    Dimension size = new Dimension(cellSize, cellSize);
    JPanel tmpPanel;
    JButton tmpButton;
    JPanel p;
    Color c = Color.BLACK;
    if(this.getValue(Option.textColor) != -1)
      c = (Color)this.getValue(Option.textColor);
    if(this.getValue(Option.fillColor) != -1)
      c = (Color)this.getValue(Option.fillColor);
    if(this.getValue(Option.lineColor) != -1)
      c = (Color)this.getValue(Option.lineColor);
    

    this.setLayout(fl);

    // Generate Transparent Button
    p = new JPanel(new GridLayout(1, 1));
    tmpButton = new JButton();
    final Color transparent = new Color(255, 255, 255, 0);
    tmpButton.setBorder(BorderFactory.createLineBorder(Color.BLACK));
    tmpButton.setBackground(transparent);
    tmpButton.setPreferredSize(new Dimension(25, 150));
    tmpButton.setText(
            "<html><center>n<br />o<br />t<br />h<br />i<br />n<br />g</center></html>");
    tmpButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        setValue((Option) values.keySet().toArray()[0], transparent);;
        fireOptionPopUpCommit(true);
      }
    });
    if(c.getAlpha() == 0)
      tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,1));
    p.add(tmpButton);
    this.add(p);

    // Generate White-Black
    p = new JPanel(new GridLayout(8, 1));
    for (int j = 0; j < 8; j++) {
      tmpPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
      tmpButton = new JButton();
      final Color buttonColor = new Color(j * offset, j * offset, j * offset);

      tmpButton.setBackground(buttonColor);
      tmpButton.setSize(size);
      tmpButton.setPreferredSize(size);
      if(c.equals(buttonColor))
        tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,1));

      tmpPanel.add(tmpButton);
      p.add(tmpPanel);

      tmpButton.addActionListener(new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
          setValue((Option) values.keySet().toArray()[0], buttonColor);
          fireOptionPopUpCommit(true);
        }
      });
    }
    this.add(p);
    int choose = 1;
    // Generate Colors
    p = new JPanel(new GridLayout(8, 5));
    if (choose == 1) {
      for (int j = 0; j < 40; j++) {
        tmpPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
        tmpButton = new JButton();
        final Color buttonColor = Color.getHSBColor((float) j / (float) 45,
                                                    0.85f, 1.0f);

        tmpButton.setBackground(buttonColor);
        tmpButton.setSize(size);
        tmpButton.setPreferredSize(size);
        if(c.equals(buttonColor))
          tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,1));
        tmpPanel.add(tmpButton);
        p.add(tmpPanel);
        tmpButton.addActionListener(new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            setValue((Option) values.keySet().toArray()[0], buttonColor);
            fireOptionPopUpCommit(true);
          }
        });
      }
      this.add(p);
    } else {
      p = new JPanel(new GridLayout(5, 25));
      int[] wert = {0, 64, 128, 192, 255};
      for (int rot = 0; rot <= 4; rot++) // Farbintensitaet rot ...
      {
        for (int gruen = 0; gruen <= 4; gruen++) // gruen ...
        {
          for (int blau = 0; blau <= 4; blau++) // blau durchlaufen
          {

            final Color buttonColor = new Color(wert[rot], wert[gruen],
                                                wert[blau]); // Farbe erzeugen
            tmpPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
            tmpButton = new JButton();

            tmpButton.setBackground(buttonColor);
            tmpButton.setSize(size);
            tmpButton.setPreferredSize(size);
            if(c.equals(buttonColor))
              tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,1));
            tmpPanel.add(tmpButton);
            p.add(tmpPanel);

            tmpButton.addActionListener(new ActionListener() {
              @Override
              public void actionPerformed(ActionEvent e) {
                Option[] o = null;
                values.keySet().toArray(o);
                setValue(o[0], buttonColor);
                fireOptionPopUpCommit(true);
              }
            });
          }
        }
      }
      this.add(p);
    }
  }

  @Override
  public Icon createIcon() {
    Icon i = null;
    for (Entry<Option, Object> entry : this.values.entrySet()) {
      if (entry.getKey() == Option.lineColor) {
        i = createLineColorIcon();
      } else {
        i = createFillColorIcon();
      }
    }
    return i;
  }

  private Icon createLineColorIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        Graphics2D g2 = (Graphics2D) g;

        g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));

        // Set chosen Color
        g2.setColor((Color) values.get(Option.lineColor));

        // Create filled shapes to see the chosen color
        Rectangle rect = new Rectangle(x, y + 12, 14, 3);
        g2.fill(rect);

        // Reset color to black and create the boarders for the filled shapes
        g2.setColor(Color.black);
        g2.draw(rect);

        // Create Pencil Body
        Polygon poly = new Polygon();
        poly.addPoint(x + 13, y + 2);
        poly.addPoint(x + 15, y + 4);
        poly.addPoint(x + 7, y + 12);
        poly.addPoint(x + 5, y + 10);
        poly.addPoint(x + 13, y + 2);
        poly.translate(-3, -2);
        g2.draw(poly);

        // Create Pencil Body-Line
        poly = new Polygon();
        poly.addPoint(x + 14, y + 3);
        poly.addPoint(x + 6, y + 11);
        poly.translate(-3, -2);
        g2.draw(poly);

        poly = new Polygon();
        poly.addPoint(x + 7, y + 12);
        poly.addPoint(x + 2, y + 15);
        poly.addPoint(x + 5, y + 10);
        poly.translate(-3, -2);
        g2.fill(poly);
      }
    };
  }

  private Icon createFillColorIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        Graphics2D g2 = (Graphics2D) g;

        g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));

        // Set chosen Color
        g2.setColor((Color) values.get(Option.fillColor));
        
        // Create filled shapes to see the chosen color
        Rectangle rect = new Rectangle(x, y + 12, 14, 3);
        g2.fill(rect);

        // Reset color to black and create the boarders for the filled shapes
        g2.setColor(Color.black);
        g2.draw(rect);

        g2.rotate(0.5);
        g2.translate(5, -6);

        // Can
        Arc2D arc = new Arc2D.Double(x, y + 6, 7, 4, 180, 180, Arc2D.OPEN);
        g2.draw(arc);

        Ellipse2D ellipse = new Ellipse2D.Double(x, y, 7, 4);
        g2.draw(ellipse);

        Line2D line = new Line2D.Double(new Point(x, y + 2), new Point(x, y + 8));
        g2.draw(line);
        line = new Line2D.Double(new Point(x + 7, y + 2),
                                 new Point(x + 7, y + 8));
        g2.draw(line);

        // Color
        arc = new Arc2D.Double(x + 4, y, 7, 10, 0, 180, Arc2D.OPEN);
        g2.fill(arc);
        arc = new Arc2D.Double(x + 4, y, 7, 10, -50, 180, Arc2D.OPEN);
        g2.fill(arc);
        arc = new Arc2D.Double(x + 4, y, 7, 10, -70, 180, Arc2D.OPEN);
        g2.fill(arc);

      }
    };
  }
}
