/**----------------------------------------------------
 * File:      LineStrengthOptionPopUp.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Popup used for choosing the linestrength
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionChangedEvent;
import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

public class LineStrengthOptionPopUp
        extends OptionPopUp {

  public LineStrengthOptionPopUp(OptionChangedEvent.Option o, Object v) {
    super(o, v);
  }

  @Override
  protected void init() {
    this.removeAll();
    GridLayout gl = new GridLayout(2, 0, 0, 0);
    this.setLayout(gl);


    FlowLayout fl = new FlowLayout(FlowLayout.LEFT, 0, 0);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(fl);
    this.add(buttonPanel);
    JPanel lblPanel = new JPanel();
    lblPanel.setLayout(fl);
    this.add(lblPanel);
    int size = 24;

    JLabel iconLabel = createIconLabel(size, 0);
    JLabel textLabel = createTextLabel(size, "<html>none</html>");
    textLabel.setFont(new Font(textLabel.getFont().getName(), Font.PLAIN, 9));
    createMouseListener(iconLabel, textLabel, -1);
    if(-1 == (int)this.getValue(Option.lineStrength))
      addBoarder(iconLabel, textLabel);
    buttonPanel.add(iconLabel);
    lblPanel.add(textLabel);

    iconLabel = createIconLabel(size, 0);
    textLabel = createTextLabel(size, "<html>hair<br />line</html>");
    textLabel.setFont(new Font(textLabel.getFont().getName(), Font.PLAIN, 9));
    createMouseListener(iconLabel, textLabel, 0);
    if(0 == (int)this.getValue(Option.lineStrength))
      addBoarder(iconLabel, textLabel);
    buttonPanel.add(iconLabel);
    lblPanel.add(textLabel);
    
    iconLabel = createIconLabel(size, 1);
    textLabel = createTextLabel(size, "1");
    createMouseListener(iconLabel, textLabel, 1);
    if(1 == (int)this.getValue(Option.lineStrength))
      addBoarder(iconLabel, textLabel);
    buttonPanel.add(iconLabel);
    lblPanel.add(textLabel);

    for (int i = 2; i <= size - 4; i = i + 2) {
      iconLabel = createIconLabel(size, i);
      textLabel = createTextLabel(size, i + "");
      createMouseListener(iconLabel, textLabel, i);
      if(i == (int)this.getValue(Option.lineStrength))
        addBoarder(iconLabel, textLabel);
      buttonPanel.add(iconLabel);
      lblPanel.add(textLabel);
    }
  }

  private JLabel createTextLabel(final int size, String text) {
    JLabel lbl = new JLabel();
    lbl.setPreferredSize(new Dimension(size, size));
    lbl.setText(text);
    lbl.setHorizontalAlignment(SwingConstants.CENTER);
    lbl.setVerticalAlignment(SwingConstants.CENTER);
    return lbl;
  }

  private JLabel createIconLabel(final int size, int lineStrength) {

    final int strength = lineStrength;

    final JLabel lbl = new JLabel();
    lbl.setPreferredSize(new Dimension(size, size));
    if (lineStrength != 0) {
      lbl.setIcon(new Icon() {
        public int getIconHeight() {
          return size;
        }

        public int getIconWidth() {
          return size;
        }

        @Override
        public void paintIcon(Component c, Graphics g, int x, int y) {
          Graphics2D g2 = (Graphics2D) g;
          g2.setColor(Color.BLACK);
          g2.setStroke(new BasicStroke(strength, BasicStroke.CAP_BUTT,
                                       BasicStroke.JOIN_MITER));
          g2.drawLine(0, getIconHeight() / 2, getIconWidth(),
                      getIconHeight() / 2);
        }
      });
    }

    return lbl;
  }

  private void createMouseListener(final JLabel label1, final JLabel label2, final int strength) {
    
    MouseListener ml = new MouseListener() {
      @Override
      public void mouseClicked(MouseEvent e) {
      }

      @Override
      public void mousePressed(MouseEvent e) {
      }

      @Override
      public void mouseReleased(MouseEvent e) {
        setValue(Option.lineStrength, strength);
        removeBoarder(label1, label2);
        fireOptionPopUpCommit(true);
      }

      @Override
      public void mouseEntered(MouseEvent e) {
        if(((int)getValue(Option.lineStrength)) != strength)
          addBoarder(label1, label2);
      }

      @Override
      public void mouseExited(MouseEvent e) {
        if(((int)getValue(Option.lineStrength)) != strength)
          removeBoarder(label1, label2);
      }
    };
    label1.addMouseListener(ml);
    label2.addMouseListener(ml);
  }

  @Override
  public Icon createIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setColor(c.getForeground());
        g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        g2.drawLine(6, 6, getIconWidth() + 1, 6);
        g2.setStroke(new BasicStroke(2, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        g2.drawLine(6, 11, getIconWidth() + 2, 11);
        g2.setStroke(new BasicStroke(3, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        g2.drawLine(6, 16, getIconWidth() + 2, 16);
      }
    };
  }
  
  private void addBoarder(JLabel lbl1, JLabel lbl2) {
    lbl1.setBorder(BorderFactory.
                createMatteBorder(1, 1, 0, 1, Color.BLACK));
    lbl2.setBorder(BorderFactory.
            createMatteBorder(0, 1, 1, 1, Color.BLACK));
  }
  
  private void removeBoarder(JLabel lbl1, JLabel lbl2) {
    lbl1.setBorder(BorderFactory.createEmptyBorder());
    lbl2.setBorder(BorderFactory.createEmptyBorder());
  }
}
