/**----------------------------------------------------
 * File:      OptionPopUp.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Abstract class that can be used to create new OptionPopUps.
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import at.hoertler.svgeditor.event.OptionPopUpCommitEvent;
import at.hoertler.svgeditor.event.OptionPopUpListener;
import at.hoertler.svgeditor.options.OptionManager;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import javax.swing.Icon;
import javax.swing.JPanel;

public abstract class OptionPopUp
        extends JPanel {

  protected ArrayList<OptionPopUpListener> popUpListener = null;
  protected HashMap<Option, Object> values = new HashMap<Option, Object>();
  //protected Option option = null;
  //protected Object value = null;

  public OptionPopUp(Option o, Object v) {
    this.popUpListener = new ArrayList<OptionPopUpListener>();
    this.values.put(o, v);
    this.init();
  }

  public OptionPopUp(HashMap<Option, Object> values) {
    this.popUpListener = new ArrayList<OptionPopUpListener>();
    this.values = values;
    this.init();
  }

  protected abstract void init();

  public abstract Icon createIcon();

  /**
   * Removes the given Listener.
   *
   * @return Result of the remove process.
   */
  public boolean removePopUpListener(OptionPopUpListener popUpListener) {
    return this.popUpListener.remove(popUpListener);
  }

  /**
   * Adds the given Listener.
   *
   * @param Result of the add process.
   */
  public boolean addPopUpListener(OptionPopUpListener popUpListener) {
    return this.popUpListener.add(popUpListener);
  }

  /**
   * Sets value for option
   *
   * @param o given option
   * @param v value to set
   */
  public void setValue(Option o, Object v) {
    if (this.values.containsKey(o)) {
      this.values.put(o, v);
    }
  }

  /**
   * Gets value for option
   *
   * @param o Option
   *
   * @return Value for specified option
   */
  public Object getValue(Option o) {
    if (this.values.containsKey(o)) {
      return this.values.get(o);
    }
    return -1;
  }

  protected void fireOptionPopUpCommit(boolean closePopUp) {
    for (OptionPopUpListener l : this.popUpListener) {
      for (Entry<Option, Object> entry : this.values.entrySet()) {
        l.commit(new OptionPopUpCommitEvent(entry.getKey(), entry.getValue(),
                                            closePopUp));
      }
    }
  }

  public void refreshValues(OptionManager om) {
    for (Entry<Option, Object> entry : this.values.entrySet()) {
      this.values.put(entry.getKey(), om.getValueForOption(entry.getKey()));
    }
    this.init();
  }
}
