/**----------------------------------------------------
 * File:      TextOptionPopUp.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Popup used for setting the text options like size, font, alignment, color, ...
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import at.hoertler.svgeditor.options.OptionManager;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.font.TextAttribute;
import java.text.AttributedString;
import java.util.ArrayList;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class TextOptionPopUp extends OptionPopUp {

    public TextOptionPopUp(HashMap<Option, Object> values) {
        super(values);
    }

    @Override
    protected void init() {
        this.removeAll();
        FlowLayout fl = new FlowLayout(FlowLayout.LEFT, 5, 0);
        GridLayout gl = new GridLayout(1,1,0,0);
        this.setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
        JPanel leftP = new JPanel();
        leftP.setLayout(new BoxLayout(leftP, BoxLayout.PAGE_AXIS));
        JPanel rightP = new JPanel();
        rightP.setLayout(new BoxLayout(rightP, BoxLayout.PAGE_AXIS));
        JPanel topP = new JPanel();
        topP.setLayout(new BoxLayout(topP, BoxLayout.PAGE_AXIS));
        JPanel midP = new JPanel();
        midP.setLayout(new BoxLayout(midP, BoxLayout.PAGE_AXIS));
        JPanel bottomP = new JPanel();
        bottomP.setLayout(new BoxLayout(bottomP, BoxLayout.PAGE_AXIS));
        
        JPanel sPanel = new JPanel(fl);
        JLabel sizeLabel = new JLabel("Size");
        sPanel.add(sizeLabel);
        leftP.add(sPanel);
        
        JPanel sizePanel = new JPanel(fl);
        createSizeChooser(sizePanel);
        leftP.add(sizePanel);
        
        JPanel fPanel = new JPanel(fl);
        JLabel fontLabel = new JLabel("Font");
        fPanel.add(fontLabel);
        leftP.add(fPanel);
        
        JPanel fontPanel = new JPanel(fl);
        createFontChooser(fontPanel);
        leftP.add(fontPanel);
        
        /*************************************/
        
        JPanel optionPanel = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
        rightP.add(optionPanel);
        createOptionChooser(optionPanel);
        
        JPanel alignmentPanel = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
        rightP.add(alignmentPanel);
        createAlignmentChooser(alignmentPanel);
        
        JPanel colorPanel = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
        createTextColorChooser(colorPanel);
        rightP.add(colorPanel);
        /*
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT,5,5));
        JButton cancelButton = new JButton("Cancel");
        buttonPanel.add(cancelButton);
        leftP.add(buttonPanel);
        
        buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT,0,0));
        JButton okButton = new JButton("Ok");
        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireOptionPopUpCommit(true);
            }
        });
        buttonPanel.add(okButton);
        rightP.add(buttonPanel);
        */
        this.add(leftP);
        this.add(rightP);
    }
    
    private void createOptionChooser(JPanel optionPanel) {
        JToggleButton bold = new JToggleButton("<html><b>B</b></html>");
        this.attachOptionToggleButtonListeners(bold, OptionManager.TextOption.bold);
        optionPanel.add(bold);
        
        JToggleButton italic = new JToggleButton("<html><i>I</i></html>");
        this.attachOptionToggleButtonListeners(italic, OptionManager.TextOption.italic);
        optionPanel.add(italic);
        
        JToggleButton underlined = new JToggleButton("<html><u>U</u></html>");
        this.attachOptionToggleButtonListeners(underlined, OptionManager.TextOption.underlined);
        optionPanel.add(underlined);
        
        JToggleButton strikethrough = new JToggleButton("<html><s>S</s></html>");
        this.attachOptionToggleButtonListeners(strikethrough, OptionManager.TextOption.strikethrough);
        optionPanel.add(strikethrough);

        for(OptionManager.TextOption to : (ArrayList<OptionManager.TextOption>)values.get(Option.textOption)) {
            switch(to) {
                case bold:
                    bold.setSelected(true);
                    break;
                case italic:
                    italic.setSelected(true);
                    break;    
                case underlined:
                    underlined.setSelected(true);
                    break;
                case strikethrough:
                    strikethrough.setSelected(true);
                    break;  
                default:
                    break;
            }
        }
    }
    
    private void attachOptionToggleButtonListeners(JToggleButton butt, final OptionManager.TextOption op) {
        butt.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                values.put(Option.textOption, setTextOption(op));
                fireOptionPopUpCommit(false);
            }
        });
    }
    
    private ArrayList<OptionManager.TextOption> setTextOption(OptionManager.TextOption o) {
        ArrayList<OptionManager.TextOption> options = (ArrayList<OptionManager.TextOption>)values.get(Option.textOption);
        if(!options.contains(o)) {
            options.add(o);
        } else {
            options.remove(o);
        }
        
        return options;
    }
    
    private void createAlignmentChooser(JPanel alignmentPanel) {
        ButtonGroup bg = new ButtonGroup();
        JToggleButton left = new JToggleButton();
        left.setIcon(new Icon() {
          @Override
          public void paintIcon(Component c, Graphics g, int x, int y) {
            Graphics2D g2 = (Graphics2D)g;
            g2.setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
            g2.setColor(Color.BLACK);
            for(int i = 0; i<16; i+=4)
              g2.drawLine(x-3, y+i, x+12, y+i);
            for(int i = 2; i<16; i+=4)
              g2.drawLine(x-3, y+i, x+7, y+i);
          }

          @Override
          public int getIconWidth() { return 10; }

          @Override
          public int getIconHeight() { return 16; }
        });
        left.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                values.put(Option.textAlignment, OptionManager.Alignment.left);
                fireOptionPopUpCommit(false);
            }
        });
        bg.add(left);
        alignmentPanel.add(left);
        
        JToggleButton center = new JToggleButton();
        center.setIcon(new Icon() {
          @Override
          public void paintIcon(Component c, Graphics g, int x, int y) {
            Graphics2D g2 = (Graphics2D)g;
            g2.setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
            g2.setColor(Color.BLACK);
            for(int i = 0; i<16; i+=4)
              g2.drawLine(x-3, y+i, x+12, y+i);
            for(int i = 2; i<16; i+=4)
              g2.drawLine(x, y+i, x+9, y+i);
          }

          @Override
          public int getIconWidth() { return 10; }

          @Override
          public int getIconHeight() { return 16; }
        });
        center.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                values.put(Option.textAlignment, OptionManager.Alignment.center);
                fireOptionPopUpCommit(false);
            }
        });
        bg.add(center);
        alignmentPanel.add(center);
        
        JToggleButton right = new JToggleButton();
        right.setIcon(new Icon() {
          @Override
          public void paintIcon(Component c, Graphics g, int x, int y) {
            Graphics2D g2 = (Graphics2D)g;
            g2.setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
            g2.setColor(Color.BLACK);
            for(int i = 0; i<16; i+=4)
              g2.drawLine(x-3, y+i, x+12, y+i);
            for(int i = 2; i<16; i+=4)
              g2.drawLine(x+2, y+i, x+12, y+i);
          }

          @Override
          public int getIconWidth() { return 10; }

          @Override
          public int getIconHeight() { return 16; }
        });
        right.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                values.put(Option.textAlignment, OptionManager.Alignment.right);
                fireOptionPopUpCommit(false);
            }
        });
        bg.add(right);
        alignmentPanel.add(right);
        
        switch((OptionManager.Alignment)values.get(Option.textAlignment)) {
            case left:
                left.setSelected(true);
                break;
            case center:
                center.setSelected(true);
                break;    
            case right:
                right.setSelected(true);
                break;
            default:
                break;
        }
        
    }
    
    private void createTextColorChooser(JPanel colorPanel) {
      Color c = (Color)this.getValue(Option.textColor);
         ButtonGroup bg = new ButtonGroup();
        int cellSize = 25;
        int offset = 255 / 7;
        Dimension size = new Dimension(cellSize, cellSize);
        JPanel tmpPanel;
        JToggleButton tmpButton;
        JPanel p;
        FlowLayout panellayout = new FlowLayout(FlowLayout.CENTER, 0, 0);
        // Generate White-Black
        p = new JPanel(new GridLayout(8, 1));
        for (int j = 0; j < 8; j++) {
            tmpPanel = new JPanel(panellayout);
            tmpButton = new JToggleButton();
            
            // Add to ButtonGroup for Toggle Feature
            bg.add(tmpButton);
            
            // Calculate black/white colors
            final Color buttonColor = new Color(j * offset, j * offset, j * offset);
            
            // Select the previous selected color
            if(buttonColor.equals(c)) {
              tmpButton.setSelected(true);
              tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,1));
            }/* else {
              tmpButton.setSelected(false);
              tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,0));
            }*/
            
            tmpButton.setBackground(buttonColor);
            tmpButton.setSize(size);
            tmpButton.setPreferredSize(size);

            tmpPanel.add(tmpButton);
            p.add(tmpPanel);

            this.attachColorToggleButtonListeners(tmpButton, buttonColor);
        }
        colorPanel.add(p);

        // Generate Colors
        p = new JPanel(new GridLayout(8, 5));
        for (int j = 0; j < 40; j++) {
            tmpPanel = new JPanel(panellayout);
            tmpButton = new JToggleButton();
            
            // Add to ButtonGroup for Toggle Feature
            bg.add(tmpButton);
            
            // Calculate colors
            final Color buttonColor = Color.getHSBColor((float) j / (float) 45, 0.85f, 1.0f);

            // Select the previous selected color
            if(buttonColor.equals(c)) {
              tmpButton.setSelected(true);
              tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,1));
            }/* else {
              tmpButton.setSelected(false);
              tmpButton.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY,0));
            }*/
            
            tmpButton.setBackground(buttonColor);
            tmpButton.setSize(size);
            tmpButton.setPreferredSize(size);

            tmpPanel.add(tmpButton);
            p.add(tmpPanel);

            this.attachColorToggleButtonListeners(tmpButton, buttonColor);
        }
        colorPanel.add(p);
    }
    
    private void attachColorToggleButtonListeners(JToggleButton butt, final Color buttColor) {
        butt.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                JToggleButton b = (JToggleButton)e.getSource();
                if(b.isSelected()) {
                    b.setBorder(new LineBorder(Color.DARK_GRAY,1));
                } else {
                    b.setBorder(new LineBorder(Color.DARK_GRAY,0));
                }
            }
        });
            
        butt.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                setValue(Option.textColor, buttColor);
                fireOptionPopUpCommit(false);
            }
        });
    }

    private void createSizeChooser(JPanel sizePanel) {
        SpinnerNumberModel model = new SpinnerNumberModel(((Integer)values.get(Option.textSize)).intValue(), 4, 30, 2); 
        int fifty = model.getNumber().intValue(); 
        JSpinner size = new JSpinner(model);
        size.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                values.put(Option.textSize, (int)((JSpinner)e.getSource()).getModel().getValue());
                fireOptionPopUpCommit(false);
            }
        });
        sizePanel.add(size);
    }
    
    private void createFontChooser(JPanel fontPanel) {
        
        JList<Font> fonts = new JList<Font>(GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts());
        fonts.setAutoscrolls(true);
        fonts.setCellRenderer(new FontCellRenderer());
        fonts.ensureIndexIsVisible(50);
        fonts.setVisibleRowCount(12);
        fonts.addListSelectionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent e) {
                values.put(Option.font, ((JList)e.getSource()).getSelectedValue());
                fireOptionPopUpCommit(false);
            }
        });
        for(int i = 0; i<GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts().length; i++) {
          Font f = GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts()[i];
          if(f.equals((Font)this.getValue(Option.font))) {
            fonts.setSelectedIndex(i);
            break;
          }
            
        }
        JScrollPane scrollPane = new JScrollPane(fonts);
        fontPanel.add(scrollPane);
    }
    
    
    @Override
    public Icon createIcon() {
        return new Icon() {
            public int getIconHeight() {
                return 16;
            }

            public int getIconWidth() {
                return 16;
            }

            @Override
            public void paintIcon(Component c, Graphics g, int x, int y) {
                Graphics2D g2 = (Graphics2D) g;
                g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                
                AttributedString as = new AttributedString("A");
                as.addAttribute(TextAttribute.FONT, new Font("Default", Font.BOLD, 15));
                as.addAttribute(TextAttribute.FOREGROUND, Color.BLACK);
                as.addAttribute(TextAttribute.WEIGHT, TextAttribute.WEIGHT_ULTRABOLD);
                as.addAttribute(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON, 0, 1);
                g2.drawString(as.getIterator(), x+0, y+10);

                as = new AttributedString("A");
                as.addAttribute(TextAttribute.FOREGROUND, Color.RED);
                as.addAttribute(TextAttribute.FONT, new Font("Default", Font.ITALIC, 13));               
                g2.drawString(as.getIterator(), x+7, y+15);
            }
        };
    }

    private class FontCellRenderer extends DefaultListCellRenderer {
        JLabel jlFontName = null;
        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            jlFontName = new JLabel();
            jlFontName.setFont(list.getFont());
            Font font = (Font)value;
            jlFontName.setText(font.getName());
            Dimension dmn = jlFontName.getPreferredSize();
            Font derivedFont = font.deriveFont((float)dmn.getHeight());
            jlFontName.setFont(derivedFont);
            jlFontName.setMinimumSize(dmn);
            jlFontName.setPreferredSize(dmn);
            jlFontName.setOpaque(true);
            if (isSelected) {
                jlFontName.setBackground(list.getSelectionBackground());
                jlFontName.setForeground(list.getSelectionForeground());
            } else {
                jlFontName.setBackground(list.getBackground());
                jlFontName.setForeground(list.getForeground());
            }
            return jlFontName;
        }
    }
}
