/**----------------------------------------------------
 * File:      ToolBarActionButton.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * This class can be used to create new ToolBarButtons which fire action events.
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.action.ActionManager;
import at.hoertler.svgeditor.event.ActionPerformedEvent;
import java.awt.BasicStroke;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.Icon;

public class ToolBarActionButton
        extends ToolBarButton {

  private ActionPerformedEvent actionEvent = null;

  public ToolBarActionButton(String name, ToolBarManager tbm, ActionPerformedEvent actev) {
    this(name, name, tbm, actev);
  }

  public ToolBarActionButton(String name, String toolTipText, ToolBarManager tbm, ActionPerformedEvent actev) {
    super(name, toolTipText, tbm);
    this.actionEvent = actev;
    this.setFocusAble(false);
    this.setIcon(this.createIcon());

    this.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {

        ActionManager.fireActionPerformedEvent(actionEvent);
      }
    });
  }

  @Override
  protected Icon createIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setColor(c.getForeground());
        g2.setStroke(new BasicStroke(2, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        g2.drawLine(x + 6, y + 6, x + 14, y + 14);
        g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        Polygon arrowHead = new Polygon();
        arrowHead.addPoint(x + 1, y + 1);
        //arrowHead.addPoint(b.x-this.arrowHeight,b.y-this.arrowLength);
        //arrowHead.addPoint(b.x+this.arrowHeight,b.y-this.arrowLength);
        arrowHead.addPoint(x + 4, y + 10);
        arrowHead.addPoint(x + 10, y + 4);
        g2.fillPolygon(arrowHead);

      }
    };
  }
}
