/**----------------------------------------------------
 * File:      ToolBarOptionButton.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * The ToolBarOptionButton handles various OptionPopUps.
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.event.OptionPopUpCommitEvent;
import at.hoertler.svgeditor.event.OptionPopUpListener;
import at.hoertler.svgeditor.options.OptionManager;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.Icon;
import javax.swing.JPopupMenu;

public class ToolBarOptionButton extends ToolBarButton implements OptionPopUpListener {
    private OptionManager optionManager = null;
    private OptionPopUp optionPopUp = null;
    private JPopupMenu popUpMenu = null;

    public ToolBarOptionButton(String name, ToolBarManager tbm, OptionManager om, OptionPopUp opu) {
        this(name, name, tbm, om, opu);
    }

    public ToolBarOptionButton(String name, String toolTipText, ToolBarManager tbm, OptionManager om, OptionPopUp opu) {
        super(name, toolTipText, tbm);
        this.optionManager = om;
        this.optionPopUp = opu;
        this.setFocusAble(false);
        this.setIcon(this.createIcon());
        this.popUpMenu = new JPopupMenu();
        this.optionPopUp.addPopUpListener(this);
        this.popUpMenu.insert(this.optionPopUp,0);
        
        this.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                optionPopUp.refreshValues(optionManager);
                popUpMenu.show(e.getComponent(), e.getX(), e.getY());
            }
        });
    }
    
    @Override
    protected Icon createIcon() {
        return this.optionPopUp.createIcon();
    }

    @Override
    public void commit(OptionPopUpCommitEvent e) {
        this.optionManager.setValueForOption(e.getOption(), e.getValue());
        if(e.isClosePopUp())
            this.popUpMenu.setVisible(false);
    }
}
