 /**----------------------------------------------------
 * File:      ActionPerformedEvent.java
 * Package:   at.hoertler.svgeditor.event
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * 
 */
package at.hoertler.svgeditor.event;

import java.awt.Point;

public class ActionPerformedEvent {

  public enum Action {
    showMagneticGrid,
    newImage,
    deselectAll, invertSelection, selectAll,
    copy, paste, cut, delete, duplicate,
    group, ungroup, 
    undo, redo,
    align,
    moveLayerUp, moveLayerDown, bringToFront, sendToBack,
    moveUp, moveDown, moveLeft, moveRight,
    takePicture, print, openImage, saveImage, saveImageAs, insertImage, 
    repaint, switchToSelection, enterGroup, leaveGroup, createHistoryPoint, closeWindow };

  public enum Alignment {

    none,
    left_up, center_up, right_up,
    left_center, center_center, right_center,
    left_bottom, center_bottom, right_bottom,
    left, center_h, right,
    top, center_v, bottom
  };
  
  private Action action;
  private Alignment alignment = Alignment.none;
  private Object value = null;

  public ActionPerformedEvent(Action a) {
    this.action = a;
  }
  
  public ActionPerformedEvent(Action a, Object value) {
    this.action = a;
    this.value = value;
  }
  
  public ActionPerformedEvent(Alignment a) {
    this.action = Action.align;
    this.alignment = a;
  }

  /**
   * @return the action
   */
  public Action getAction() {
    return action;
  }

  /**
   * @param action the action to set
   */
  public void setAction(Action action) {
    this.action = action;
  }

  /**
   * @return the alignment
   */
  public Alignment getAlignment() {
    return alignment;
  }

  /**
   * @param alignment the alignment to set
   */
  public void setAlignment(Alignment alignment) {
    this.alignment = alignment;
  }

  /**
   * @return the value
   */
  public Object getValue() {
    return value;
  }

  /**
   * @param value the value to set
   */
  public void setValue(Object value) {
    this.value = value;
  }
}
