 /**----------------------------------------------------
 * File:      CanvasStateEvent.java
 * Package:   at.hoertler.svgeditor.event
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * 
 */
package at.hoertler.svgeditor.event;

import at.hoertler.svgeditor.common.HelperFunction;
import at.hoertler.svgeditor.svg.elements.SVGElement;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.Vector;

public class CanvasStateEvent {
  private Point startPoint = new Point(0,0);
  private Point endPoint = new Point(0,0);
  private Point delta = new Point(0,0);
  private Vector<SVGElement> selectedElements = null;
  private Rectangle boundingBox = null;
  private MouseEvent mouseEvent = null;


  public CanvasStateEvent(Point start, Point end, MouseEvent me, Vector<SVGElement> elems) {
    this.selectedElements = elems;
    if(start!=null)
      this.startPoint = start;
    if(end!=null)
      this.endPoint = end;
    this.mouseEvent = me;
    this.delta = HelperFunction.calculateTranslationPoint(start, end);
    this.boundingBox = new Rectangle(0,0,0,0);
    Point xy = new Point(Integer.MAX_VALUE, Integer.MAX_VALUE);
    Dimension d = new Dimension(0,0);
    for(SVGElement e : this.selectedElements) {
      if(xy.x>e.getBoundingBox().x)
        xy.x = e.getBoundingBox().x;
      if(xy.y>e.getBoundingBox().y)
        xy.y = e.getBoundingBox().y;
      if(d.width+xy.x < e.getBoundingBox().width+e.getBoundingBox().x)
        d.width = e.getBoundingBox().width+e.getBoundingBox().x-xy.x;
      if(d.height+xy.y < e.getBoundingBox().height+e.getBoundingBox().y)
        d.height = e.getBoundingBox().height+e.getBoundingBox().y-xy.y;
    }
    if(!this.selectedElements.isEmpty()) {
      this.boundingBox.setLocation(xy);
      this.boundingBox.setSize(d);
    }
  }

  /**
   * @return the startPoint
   */
  public Point getStartPoint() {
    return startPoint;
  }

  /**
   * @param startPoint the startPoint to set
   */
  public void setStartPoint(Point startPoint) {
    this.startPoint = startPoint;
  }

  /**
   * @return the endPoint
   */
  public Point getEndPoint() {
    return endPoint;
  }

  /**
   * @param endPoint the endPoint to set
   */
  public void setEndPoint(Point endPoint) {
    this.endPoint = endPoint;
  }

  /**
   * @return the delta
   */
  public Point getDelta() {
    return delta;
  }

  /**
   * @param delta the delta to set
   */
  public void setDelta(Point delta) {
    this.delta = delta;
  }

  /**
   * @return the selectedElements
   */
  public Vector<SVGElement> getSelectedElements() {
    return selectedElements;
  }

  /**
   * @param selectedElements the selectedElements to set
   */
  public void setSelectedElements(Vector<SVGElement> selectedElements) {
    this.selectedElements = selectedElements;
  }

  /**
   * @return the boundingBox
   */
  public Rectangle getBoundingBox() {
    return boundingBox;
  }

  /**
   * @param boundingBox the boundingBox to set
   */
  public void setBoundingBox(Rectangle boundingBox) {
    this.boundingBox = boundingBox;
  }

  /**
   * @return the mouseEvent
   */
  public MouseEvent getMouseEvent() {
    return mouseEvent;
  }

  /**
   * @param mouseEvent the mouseEvent to set
   */
  public void setMouseEvent(MouseEvent mouseEvent) {
    this.mouseEvent = mouseEvent;
  }

}