 /**----------------------------------------------------
 * File:      SVGSelection.java
 * Package:   at.hoertler.svgeditor.svg.elements
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * Represents an SVGSelection-Element.
 */
package at.hoertler.svgeditor.svg.elements;

import at.hoertler.svgeditor.action.ActionManager;
import at.hoertler.svgeditor.common.HelperFunction;
import at.hoertler.svgeditor.data.ElementManager;
import at.hoertler.svgeditor.event.ActionPerformedEvent;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import javax.swing.Icon;

public class SVGSelection
        extends SVGElement {

  @Override
  public Point getTranslationPointForHandle(int handleIndex) {
    return this.getHandles().get((handleIndex+1)%2);
  }

  /**
   * Specifies what has been hit.
   */
  public enum Hit {

    /**
     * Nothing has been hit.
     */
    NOTHING,
    /**
     * Handle has been hit.
     */
    HANDLE,
    /**
     * Already selected element has been hit.
     */
    SELECTED_ELEMENT,
    /**
     * Not selected element has been hit.
     */
    ELEMENT
  };

  @Override
  public boolean hit(Rectangle rect) {
    throw new UnsupportedOperationException("Not supported yet.");
  }
  private Hit mode = null;
  private int handleIndex = -1;
  public Point grabPoint = null;
  public Point releasePoint = null;
  private SVGElement activeElement = null;
  private ElementManager elementManager = null;
  private boolean duplicate = false;
  private boolean keepCtrlDown = false;
  Point transP1 = null;
  Point transP2 = null;

  public SVGSelection() {
    this.setLineColor(Color.black);
    this.getHandles().add(new Point(-1,-1));
    this.getHandles().add(new Point(-1,-1));
  }

  /**
   * @param elementManager the ElementManager to set
   */
  public void setElementManager(ElementManager elementManager) {
    this.elementManager = elementManager;
  }

  @Override
  public SVGElement getInstance() {
    SVGSelection sel = new SVGSelection();
    sel.setElementManager(this.elementManager);
    return sel;
  }

  @Override
  public void paint(Graphics g) {

    // Set Graphics2D Object to be used in other operations.
    this.setGraphics((Graphics2D) g);
    this.getGraphics().setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                                 BasicStroke.JOIN_BEVEL, 0,
                                                 new float[] {3, 1}, 0));


    // Set the Line Color
    this.getGraphics().setColor(Color.black);

    // Draw Shape
    this.getGraphics().draw(
            new Rectangle2D.Double(this.getBoundingBox().x,
                                   this.getBoundingBox().y,
                                   this.getBoundingBox().width,
                                   this.getBoundingBox().height));
  }

  @Override
  public void paint(Graphics g, int zoomFactor) {
    if (this.mode == Hit.NOTHING && 
        this.getBoundingBox().width>1 && 
        this.getBoundingBox().height>1) { // Rendering only neccessary if Selection Box should be rendered
      // Set Graphics2D Object to be used in other operations.
      this.setGraphics((Graphics2D) g);
      this.getGraphics().setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                                   BasicStroke.JOIN_BEVEL, 0,
                                                   new float[] {3, 1}, 0));


      // Set the Line Color
      this.getGraphics().setColor(Color.black);

      // Draw Shape
      this.getGraphics().
              draw(
              new Rectangle2D.Double(this.getBoundingBox().x * zoomFactor,
                                     this.getBoundingBox().y * zoomFactor,
                                     this.getBoundingBox().width * zoomFactor,
                                     this.getBoundingBox().height * zoomFactor));
    }
  }

  @Override
  public Icon getIcon() {
    return new Icon() {
      public int getIconHeight() {
        return 16;
      }

      public int getIconWidth() {
        return 16;
      }

      @Override
      public void paintIcon(Component c, Graphics g, int x, int y) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setColor(Color.BLACK);
        g2.setStroke(new BasicStroke(2, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        g2.drawLine(x + 7, y + 7, x + 14, y + 14);
        g2.setStroke(new BasicStroke(1, BasicStroke.CAP_BUTT,
                                     BasicStroke.JOIN_MITER));
        g2.drawLine(x + 4, y + 10, x + 10, y + 4);
        g2.drawLine(x + 4, y + 10, x + 1, y + 1);
        g2.drawLine(x + 10, y + 4, x + 1, y + 1);
      }
    };
  }

  /**
   * @return the activeElement
   */
  public SVGElement getActiveElement() {
    return activeElement;
  }

  /**
   * @param activeElement the activeElement to set
   */
  public void setActiveElement(SVGElement activeElement) {
    this.activeElement = activeElement;
  }

  @Override
  public void moveHandle(int handleIndex, Point p, MouseEvent e) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public boolean validate() {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public String exportToSVG() {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleKeyPressed(KeyEvent e) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleKeyReleased(KeyEvent e) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleKeyTyped(KeyEvent e) {
    throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleMouseMoved(MouseEvent e) {
    //throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleMouseEntered(MouseEvent e) {
    //throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleMouseExited(MouseEvent e) {
    //throw new UnsupportedOperationException("Not supported yet.");
  }

  @Override
  public void handleMousePressed(MouseEvent e) {
    transP1 = null;
    transP2 = null;
          
    // if CTRL-Down -> go to Duplication Mode!!
    if (e.isControlDown())
      this.duplicate = true;
    else
      this.duplicate = false;

    this.getHandles().set(0, e.getPoint());
    this.getHandles().set(1, e.getPoint());
    this.setActiveElement(this.elementManager.getHitElement(e.getPoint()));
    for (SVGElement elem : this.elementManager.getSelectedElements())
      if (elem.hitHandle(e.getPoint())) {
        this.setActiveElement(elem);
        break;
      }

    if (getActiveElement() == null) { // Nothing hit
      this.mode = Hit.NOTHING;
      this.elementManager.deselectAll();
    } else { // Element Hit
      if (getActiveElement().isSelected()) { // Selected Element Hit
        this.mode = Hit.SELECTED_ELEMENT;

        int index = getActiveElement().getHandleIndex(e.getPoint());
        if (index != -1) { // Handle of Selected Element Hit
          this.mode = Hit.HANDLE;
          this.handleIndex = index;
          // if CTRL-Down when handle is hit, turn off Duplication Mode because we want to scale the element and keep its proportion
          this.duplicate = false;
          
          // if CTRL is down when handle in ExtendedEdit-Mode is hit, save the state.
          if(this.getActiveElement().isExtendedEditMode() && e.isControlDown())
            this.keepCtrlDown = true;
          else {
            // set two points to calculate the intersection line to keep the elements proportions while moving the handle.
            // these points are used when CTRL-Down while dragging the handle.
            this.transP1 = (Point) this.getActiveElement().getHandles().get(
                this.handleIndex).clone();
            this.transP2 = this.activeElement.getTranslationPointForHandle(
                this.handleIndex);
          }
        }
      } else { // Body of not yet selected Element Hit
        this.mode = Hit.ELEMENT;
        if(!e.isControlDown()) // Multiselect: Dont deselect elements
          this.elementManager.deselectAll();
        
        this.elementManager.setSelectedElement(getActiveElement());
      }
      this.grabPoint = e.getPoint();
      this.releasePoint = e.getPoint();
    }
  }

  @Override
  public void handleMouseDragged(MouseEvent e) {

    Point deltaPoint = new Point(0, 0);
    Point newHandle = new Point(0, 0);
    if (this.grabPoint != null && this.releasePoint != null)
      deltaPoint = HelperFunction.
              calculateTranslationPoint(this.releasePoint, e.getPoint());

    switch (this.mode) {
      case NOTHING:
        this.releasePoint = e.getPoint();
        this.getHandles().set(1, e.getPoint());
        this.updateBoundingRect(this.getHandles());
        break;
      case ELEMENT:
        this.releasePoint = e.getPoint();
        this.getActiveElement().translate(deltaPoint);
        break;
      case SELECTED_ELEMENT:
        this.releasePoint = e.getPoint();
        if (this.duplicate) {
          this.duplicate = false;
          this.elementManager.duplicateSelectedElements();
          //this.activeElement=null;
        }
        this.elementManager.moveSelectedElements(deltaPoint);
        break;
      case HANDLE:
        if (e.isControlDown() && !this.getActiveElement().isExtendedEditMode()) {
          /*
           * If CTRL-Down while moving an handle, the active element is scaled
           * and its proportion are kept. A translation line is calculated and
           * checked for intersection points. The handle can be moved to every
           * point on this line and the elements proportions are guaranteed at
           * all times.
           */
          Line2D transLine = new Line2D.Float(transP1, transP2);
          Line2D yL = new Line2D.Float(0, e.getPoint().y,
                                       e.getPoint().x + transP2.x,
                                       e.getPoint().y);
          Line2D xL = new Line2D.Float(e.getPoint().x, 0, e.getPoint().x, e.
                  getPoint().y + transP2.y);
          Point intersY = HelperFunction.intersectLines(transLine, yL);
          Point intersX = HelperFunction.intersectLines(transLine, xL);
          Point inters = (transP1.distance(intersY) > transP1.distance(intersX)) ? intersY : intersX;
          deltaPoint = HelperFunction.
                  calculateTranslationPoint((Point) this.getActiveElement().
                  getHandles().get(this.handleIndex), inters);
          this.releasePoint.translate(deltaPoint.x, deltaPoint.y);

        } else {
           if(keepCtrlDown)
              e = new MouseEvent((Component)e.getSource(), e.getID(), e.getWhen(), MouseEvent.BUTTON1_MASK+MouseEvent.CTRL_MASK, e.getX(), e.getY(), e.getClickCount(), e.isPopupTrigger(), e.getButton());
           else
             e = new MouseEvent((Component)e.getSource(), e.getID(), e.getWhen(), MouseEvent.BUTTON1_MASK, e.getX(), e.getY(), e.getClickCount(), e.isPopupTrigger(), e.getButton());
          this.releasePoint = e.getPoint();
        }
          
        this.getActiveElement().moveHandle(this.handleIndex, deltaPoint, e);

        break;
      default:
        return;
    }
  }

  @Override
  public void handleMouseReleased(MouseEvent e) {
    this.getHandles().set(1, e.getPoint());
    switch (this.mode) {
      case NOTHING:
        this.getHandles().set(1, e.getPoint());
        this.updateBoundingRect(this.getHandles());
        this.elementManager.setSelectedElements(this.elementManager.
                getTotallyHitElements(this.getBoundingBox()));
        break;
      case ELEMENT:
        
      case SELECTED_ELEMENT:
        if(this.grabPoint.distance(this.releasePoint)>2) 
          ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                    ActionPerformedEvent.Action.createHistoryPoint));
        break;
      case HANDLE:
        if(keepCtrlDown)
              e = new MouseEvent((Component)e.getSource(), e.getID(), e.getWhen(), MouseEvent.BUTTON1_MASK+MouseEvent.CTRL_MASK, e.getX(), e.getY(), e.getClickCount(), e.isPopupTrigger(), e.getButton());
           else
             e = new MouseEvent((Component)e.getSource(), e.getID(), e.getWhen(), MouseEvent.BUTTON1_MASK, e.getX(), e.getY(), e.getClickCount(), e.isPopupTrigger(), e.getButton());
        /*
         * deSelect + select to calculate Normal and Editmode in order to render
         * the element only in a valid state. if the element is not valid when
         * mouse is released, the last valid state is rendered and the changes
         * are discarded
         */
        this.getActiveElement().moveHandleFinished(this.handleIndex, e);
        //this.getActiveElement().deSelect();
        //this.getActiveElement().select();
        if(this.grabPoint.distance(this.releasePoint)>2)
          ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                    ActionPerformedEvent.Action.createHistoryPoint));
        break;
    }
    this.getHandles().get(0).setLocation(0, 0);
    this.getHandles().get(1).setLocation(1, 1);
    this.updateBoundingRect(this.getHandles());
  }

  @Override
  public void handleMouseClicked(MouseEvent e) {
    if(this.activeElement!=null) {
      if(e.getClickCount()>=2) {
        if(this.activeElement.getClass().isAssignableFrom(SVGComposite.class)) {
          ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(ActionPerformedEvent.Action.enterGroup, this.activeElement));
          //this.elementManager.startEditComposite((SVGComposite)this.activeElement);
          return;
        }
      } else if(e.isControlDown()) { // Multi-select with Ctrl-Down
        if(this.activeElement.isSelected() && this.mode == Hit.SELECTED_ELEMENT)
          this.activeElement.deSelect();
        else if(this.mode == Hit.ELEMENT)
          this.activeElement.select();
      }
      
      this.activeElement.handleMouseClicked(e);
    } else {
      if(this.elementManager.isInCompositeEditMode() && e.getClickCount()>=2) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(ActionPerformedEvent.Action.leaveGroup));
        //this.elementManager.endEditComposite();
      }
    }
  }
  /*
   * @Override public void updateBoundingRect(Vector<Point> handles) { Point a =
   * new Point(Math.min(this.getHandles().get(0).x, this.getHandles().get(1).x),
   * Math.min(this.getHandles().get(0).y, this.getHandles().get(1).y)); //a =
   * calcZoom(a); Point b = new Point(Math.max(this.getHandles().get(0).x,
   * this.getHandles().get(1).x), Math.max(this.getHandles().get(0).y,
   * this.getHandles().get(1).y)); //b = calcZoom(b); Dimension d = new
   * Dimension(b.x - a.x, a.y - b.y);
   *
   *
   * // Create Shape this.getBoundingRect().setRect(a.x, a.y,
   * Math.abs(d.width), Math.abs(d.height)); }
   */
}
